
enum AddrType {
    //% block="0x19"
    ADDR_0x19,
}


enum EnvironmentType {
    //% block="标准颗粒物"
    STANDARD,
    //% block="大气"
    ATMOSPHERE
}

enum ParticleType {
    //% block="PM2.5"
    PM2_5,
    //% block="PM1.0"
    PM1_0,
    //% block="PM10"
    PM10
}

enum ParticleDiameterType {
    //% block="0.3um"
    PARTICLENUM_0_3,
    //% block="0.5um"
    PARTICLENUM_0_5,
    //% block="1.0um"
    PARTICLENUM_1_0,
    //% block="2.5um"
    PARTICLENUM_2_5,
    //% block="5.0um"
    PARTICLENUM_5_0,
    //% block="10um"
    PARTICLENUM_10
}


//% color="#6b9957" iconWidth=35 iconHeight=35
namespace AirQualitySensor {

    //% block="初始化设备直到成功 I2C地址为 [ADDR]" blockType="command"
    //% ADDR.shadow="dropdown" ADDR.options="AddrType" ADDR.defl="ADDR_0x19"
    export function begin(parameter: any, block: any) {
        Generator.addInclude("DFRobot_AirQualitySensor", '#include "DFRobot_AirQualitySensor.h"');
        Generator.addObject(`particle`, `DFRobot_AirQualitySensor_I2C`, `particle(&Wire, 0x19);`);
        Generator.addCode([
            `while(!particle.begin())`,
            `{`,
            `	delay(500);`,
            `}`,
        ].join('\n\t'));
    }

    //% block="读取 [ENVIRONMENT] 环境下 [KAIGUAN] 颗粒物浓度（ug/m³）" blockType="reporter"
    //% ENVIRONMENT.shadow="dropdown" ENVIRONMENT.options="EnvironmentType" ENVIRONMENT.defl="STANDARD"
    //% KAIGUAN.shadow="dropdown" KAIGUAN.options="ParticleType" KAIGUAN.defl="PM_2_5"
    export function readConcentration(parameter: any, block: any) {
        let environment = parameter.ENVIRONMENT.code;
        let kaiguan = parameter.KAIGUAN.code;
        Generator.addInclude("DFRobot_AirQualitySensor", '#include "DFRobot_AirQualitySensor.h"');
        Generator.addObject(`particle`, `DFRobot_AirQualitySensor_I2C`, `particle(&Wire, 0x19);`);
        Generator.addCode([`particle.gainParticleConcentration_ugm3(PARTICLE_${kaiguan}_${environment})`, Generator.ORDER_UNARY_POSTFIX]);
    }

    //% block="读取每0.1L空气中直径在 [PARTICLEDIAMETER] 的颗粒物个数" blockType="reporter"
    //% PARTICLEDIAMETER.shadow="dropdown" PARTICLEDIAMETER.options="ParticleDiameterType" PARTICLEDIAMETER.defl="D_0_3"
    export function readCount(parameter: any, block: any) {
        let particlediameter = parameter.PARTICLEDIAMETER.code;
        Generator.addInclude("DFRobot_AirQualitySensor", '#include "DFRobot_AirQualitySensor.h"');
        Generator.addObject(`particle`, `DFRobot_AirQualitySensor_I2C`, `particle(&Wire, 0x19);`);
        Generator.addCode([`particle.gainParticleNum_Every0_1L(${particlediameter}_UM_EVERY0_1L_AIR)`, Generator.ORDER_UNARY_POSTFIX]);
    }

    //% block="获取传感器版本号" blockType="reporter"
    export function readDeviceVersion(parameter: any, block: any) {
        Generator.addInclude("DFRobot_AirQualitySensor", '#include "DFRobot_AirQualitySensor.h"');
        Generator.addObject(`particle`, `DFRobot_AirQualitySensor_I2C`, `particle(&Wire, 0x19);`);
        Generator.addCode(["particle.gainVersion()", Generator.ORDER_UNARY_POSTFIX]);
    }
}
